package com.ibm.commerce.payments.plugin.applepay.beans;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.math.BigDecimal;
import java.rmi.RemoteException;

import com.ibm.commerce.applepay.payment.commands.ApplePayOrderUtils;
import com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmd;
import com.ibm.commerce.applepay.payment.objects.PKPaymentToken;
import com.ibm.commerce.command.CommandContext;
import com.ibm.commerce.command.CommandContextImpl;
import com.ibm.commerce.command.CommandFactory;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.edp.utils.Constants;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.order.facade.datatypes.OrderType;
import com.ibm.commerce.payments.plugin.Batch;
import com.ibm.commerce.payments.plugin.Credit;
import com.ibm.commerce.payments.plugin.FinancialTransaction;
import com.ibm.commerce.payments.plugin.InvalidDataException;
import com.ibm.commerce.payments.plugin.Payment;
import com.ibm.commerce.payments.plugin.PaymentInstruction;
import com.ibm.commerce.payments.plugin.PluginConfiguration;
import com.ibm.commerce.payments.plugin.PluginContext;
import com.ibm.commerce.payments.plugin.PluginException;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;


/**
 * Bean implementation class for Enterprise Bean: ApplePay
 */
public class ApplePayBean implements javax.ejb.SessionBean {

	static final long serialVersionUID = 3206093459760846163L;
	private javax.ejb.SessionContext mySessionCtx;
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}
	
	public static final String CLASS_NAME = ApplePayBean.class.getName();
	public static final long TRACE_COMPONENT = ECTraceIdentifiers.COMPONENT_PPC_PLUGIN;
	
	public static final String MERCHANT_IDENTIFIER = "merchantIdentifier";
	public static final String DOMAIN_NAME = "domainName";
	public static final String DISPLAY_NAME = "displayName";
	public static final String APPLE_PAY_PLUGIN_NAME = "ApplePay";
	public static final String APPLE_PAY_PAYMENT_TOKEN = "paymentToken";
	//~ Methods ------------------------------------------------------------------------------------
	
	/**
	 * <p>
	 * This method gets the translated error message associated with the particular error key.
	 * </p>
     * @param pluginContext The plug-in context to be used in this request.
	 * @param messageKey The error unique key.
	 * @return The translated error message if available; otherwise the <code>messageKey</code>
     *         itself.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <samp>Javadocs</samp>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	public String getMessage(
	    PluginContext pluginContext,
	    String        messageKey)
		throws PluginException, RemoteException{
		return messageKey;
	}

	/**
	 * <p>
	 * This method approves an amount against a particular payment instruction,
	 * it passes the payment token from ApplePay to ProcessApplePayPaymentCmd for processing.
     * </p>
	 *
     * @param pluginContext The plug-in context to be used in this request.
	 * @param financialTransaction The details of the transaction to be processed.
     * @param retry Indicates if the same transaction has been tried before by the 
     *        Payment Plug-in Controller.
     * @return The <code>FinancialTransaction</code> with the results of the transaction.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <samp>Javadocs</samp>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	@SuppressWarnings("deprecation")
	public FinancialTransaction approve(
	    PluginContext        pluginContext,
	    FinancialTransaction approveTransaction,
	    boolean              retry)
		throws PluginException {
        final String METHOD_NAME = "approve(pluginContext, approveTransaction, retry)";

		Object[] params = {pluginContext, approveTransaction, retry};
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME, params);

		try{
			PaymentInstruction pi = approveTransaction.getPayment().getPaymentInstruction(); 
			Integer storeId = Integer.valueOf(pi.getStore());
			PKPaymentToken paymentToken = ApplePayOrderUtils.composePaymentToken(pi.getExtendedData().getExtendedDataAsHashMap());
			
			String paymentSystem = pi.getPaymentSystemName();
			String paymentConfigGroup = pluginContext.getPaymentConfigurationGroupId();
			TypedProperty merchantInfo = ApplePayOrderUtils.getMerchantInfo(storeId, paymentSystem, paymentConfigGroup);
			OrderType orderNoun = ApplePayOrderUtils.getOrderNoun(pi.getOrderId(), false);
			
			//Invoke ProcessApplePayPaymentCmd to process the payment token from ApplePay.
			ProcessApplePayPaymentCmd processApplePaypaymentCmd = (ProcessApplePayPaymentCmd) CommandFactory.createCommand(ProcessApplePayPaymentCmd.NAME, storeId);
			CommandContext cmdCtx = new CommandContextImpl();
			cmdCtx.setStoreId(storeId);
			cmdCtx.setCommand(processApplePaypaymentCmd);
			
			processApplePaypaymentCmd.setPaymentAction(Constants.EDP_BACKEND_ACTION_APPROVE);
			processApplePaypaymentCmd.setPaymentToken(paymentToken);
			processApplePaypaymentCmd.setFinancialTransaction(approveTransaction);
			processApplePaypaymentCmd.setMerchantInfo(merchantInfo);
			processApplePaypaymentCmd.setOrderNoun(orderNoun);
			processApplePaypaymentCmd.setCommandContext(cmdCtx);
			processApplePaypaymentCmd.execute();
			
			//Get the result from ProcessApplePayPaymentCmd, and set back to PPC framework.
			approveTransaction = processApplePaypaymentCmd.getFinancialTransaction();
			
		}catch(ECException e){
			throw new PluginException(e);
		}finally{
			ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		}

        return approveTransaction;		
	}

	/**
	 * <p>
	 * This method deposits a payment without a prior approval.  Also called a "sale" transaction or
	 * "authorization with capture."
	 * </p>
     * @param pluginContext The plug-in context to be used in this request.
     * @param financialTransaction The details of the transaction to be processed.
     * @param retry Indicates if the same transaction has been tried before by the 
     *        Payment Plug-in Controller.
     * @return The <code>FinancialTransaction</code> with the results of the transaction.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <samp>Javadocs</samp>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	public FinancialTransaction approveAndDeposit(
	    PluginContext        pluginContext,
	    FinancialTransaction financialTransaction,
	    boolean              retry)
		throws PluginException, RemoteException{
        final String METHOD_NAME = "approveAndDeposit(pluginContext, financialTransaction, retry)";

		Object[] params = {pluginContext, financialTransaction, retry};
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME, params);

		try{
			PaymentInstruction pi = financialTransaction.getPayment().getPaymentInstruction(); 
			Integer storeId = Integer.valueOf(pi.getStore());
			PKPaymentToken paymentToken = ApplePayOrderUtils.composePaymentToken(pi.getExtendedData().getExtendedDataAsHashMap());
			
			String paymentSystem = pi.getPaymentSystemName();
			String paymentConfigGroup = pluginContext.getPaymentConfigurationGroupId();
			TypedProperty merchantInfo = ApplePayOrderUtils.getMerchantInfo(storeId, paymentSystem, paymentConfigGroup);
			OrderType orderNoun = ApplePayOrderUtils.getOrderNoun(pi.getOrderId(), false);
			
			//Invoke ProcessApplePayPaymentCmd to process the payment token from ApplePay.
			ProcessApplePayPaymentCmd processApplePaypaymentCmd = (ProcessApplePayPaymentCmd) CommandFactory.createCommand(ProcessApplePayPaymentCmd.NAME, storeId);
			CommandContext cmdCtx = new CommandContextImpl();
			cmdCtx.setStoreId(storeId);
			cmdCtx.setCommand(processApplePaypaymentCmd);
			
			processApplePaypaymentCmd.setPaymentAction(Constants.EDP_BACKEND_ACTION_APPROVE_AND_DEPOSIT);
			processApplePaypaymentCmd.setPaymentToken(paymentToken);
			processApplePaypaymentCmd.setFinancialTransaction(financialTransaction);
			processApplePaypaymentCmd.setMerchantInfo(merchantInfo);
			processApplePaypaymentCmd.setOrderNoun(orderNoun);
			processApplePaypaymentCmd.setCommandContext(cmdCtx);
			processApplePaypaymentCmd.execute();
			
			//Get the result from ProcessApplePayPaymentCmd, and set back to PPC framework.
			financialTransaction = processApplePaypaymentCmd.getFinancialTransaction();
			
		}catch(ECException e){
			throw new PluginException(e);
		}finally{
			ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		}

        return financialTransaction;		
	}

	/**
	 * <p>
	 * This method checks the existence of all required parameters in a {@link PaymentInstruction} and its
     * syntactic correctness.
	 * </p>
     * @param pluginContext The plug-in context to be used in this request.
	 * @param paymentInstruction Contains the payment method details required for the validation.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <samp>Javadocs</samp>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
     * 
     * @see #validatePaymentInstruction(PluginContext, PaymentInstruction).
	 */
	public void checkPaymentInstruction(
	    PluginContext      pluginContext,
	    PaymentInstruction paymentInstruction)
		throws PluginException, RemoteException{
		
		
	}

	/**
	 * <p>
	 * This method credits an amount against a specific {@link PaymentInstruction PaymentInstruction}. 
     * </p>
     * 
     * <p>
     * This method supports dependent (associated with a prior <code>deposit</code>)
     * and independent (not associated with a prior <code>deposit</code>) credits.  The 
     * {@link PaymentInstruction PaymentInstruction} associated with the {@link Credit Credit} 
     * in the <code>FinancialTransaction</code> allows the plug-in to determine which type 
     * of credit this transaction represents.
	 * </p>
	 *
	 * <p>
	 * Independent Credit:  the {@link PaymentInstruction PaymentInstruction} referenced by
     * {@link Credit Credit} will not have any {@link Payment Payment}s associated with it.
	 * </p>
	 *
	 * <p>
	 * Dependent Credit:  {@link PaymentInstruction PaymentInstruction} referenced by
     * {@link Credit Credit} will have at least one {@link Payment Payment} associated with it.
     * If the associated <code>Payment</code> does not have an actual approved 
     * and deposited amount, this is not a real dependent credit transaction. 
	 * </p>
	 *
     * @param pluginContext The plug-in context to be used in this request.
     * @param financialTransaction The details of the transaction to be processed.
     * @param retry Indicates if the same transaction has been tried before by the 
     *        Payment Plug-in Controller.
     * @return The <code>FinancialTransaction</code> with the results of the transaction.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <samp>Javadocs</samp>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	public FinancialTransaction credit(
	    PluginContext        pluginContext,
	    FinancialTransaction financialTransaction,
	    boolean              retry)
		throws PluginException, RemoteException{
		
		return financialTransaction;
	}

	/**
	 * <p>
	 * This method deposits an amount against a previously approved <code>Payment<code>. 
     * </p>
     * 
     * <p>
     * This is typically invoked after the merchant has shipped the goods and is now 
     * entitled to receive payment.
	 * </p>
	 *
	 * <p>
	 * This method should be implemented if the method 
     * <samp>approve(PluginContext, FinancialTransaction, boolean)</samp> is implemented. 
     * This method is typically implemented for credit cards, gift cards or 
     * similar payment methods where funds are reserved and captured later on. For example: 
     * Merchants processing credit cards will often approve the total ordered amount and 
     * capture when packages are shipped.
	 * </p>
	 *
     * @param pluginContext The plug-in context to be used in this request.
     * @param financialTransaction The details of the transaction to be processed.
     * @param retry Indicates if the same transaction has been tried before by the 
     *        Payment Plug-in Controller.
     * @return The <code>FinancialTransaction</code> with the results of the transaction.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <samp>Javadocs</samp>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	@SuppressWarnings("deprecation")
	public FinancialTransaction deposit(
	    PluginContext        pluginContext,
	    FinancialTransaction financialTransaction,
	    boolean              retry)
		throws PluginException {
        final String METHOD_NAME = "deposit(pluginContext, financialTransaction, retry)";

		Object[] params = {pluginContext, financialTransaction, retry};
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME, params);

		try{
			PaymentInstruction pi = financialTransaction.getPayment().getPaymentInstruction(); 
			Integer storeId = Integer.valueOf(pi.getStore());
			PKPaymentToken paymentToken = ApplePayOrderUtils.composePaymentToken(pi.getExtendedData().getExtendedDataAsHashMap());
			
			String paymentSystem = pi.getPaymentSystemName();
			String paymentConfigGroup = pluginContext.getPaymentConfigurationGroupId();
			TypedProperty merchantInfo = ApplePayOrderUtils.getMerchantInfo(storeId, paymentSystem, paymentConfigGroup);
			OrderType orderNoun = ApplePayOrderUtils.getOrderNoun(pi.getOrderId(), false);
			
			//Invoke ProcessApplePayPaymentCmd to process the payment token from ApplePay.
			ProcessApplePayPaymentCmd processApplePaypaymentCmd = (ProcessApplePayPaymentCmd) CommandFactory.createCommand(ProcessApplePayPaymentCmd.NAME, storeId);
			CommandContext cmdCtx = new CommandContextImpl();
			cmdCtx.setStoreId(storeId);
			cmdCtx.setCommand(processApplePaypaymentCmd);
			
			processApplePaypaymentCmd.setPaymentAction(Constants.EDP_BACKEND_ACTION_DEPOSIT);
			processApplePaypaymentCmd.setPaymentToken(paymentToken);
			processApplePaypaymentCmd.setFinancialTransaction(financialTransaction);
			processApplePaypaymentCmd.setMerchantInfo(merchantInfo);
			processApplePaypaymentCmd.setOrderNoun(orderNoun);
			processApplePaypaymentCmd.setCommandContext(cmdCtx);
			processApplePaypaymentCmd.execute();
			
			//Get the result from ProcessApplePayPaymentCmd, and set back to PPC framework.
			financialTransaction = processApplePaypaymentCmd.getFinancialTransaction();
			
		}catch(ECException e){
			throw new PluginException(e);
		}finally{
			ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		}

        return financialTransaction;		
	}

	/**
	 * <p>
	 * This method cancels a payment that has been previously approved.
	 * </p>
	 *
	 * <p>
	 * The amount can be partial if the plug-in back-end supports it.  
     * If the plug-in does not support partial amounts then the 
     * {@link InvalidDataException} should be thrown.
	 * </p>
	 * 
     * <p>
     * A <code>reverse approval</code> transaction is only requested against a plug-in if a previous
     * <code>approve</code> transaction has been requested and is either in 
     * {@link FinancialTransaction#STATE_SUCCESS} or {@link FinancialTransaction#STATE_FAILED}.  
     * </p>
     * 
     * @param pluginContext The plug-in context to be used in this request.
     * @param financialTransaction The details of the transaction to be processed.
     * @param retry Indicates if the same transaction has been tried before by the 
     *        Payment Plug-in Controller.
     * @return The <code>FinancialTransaction</code> with the results of the transaction.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <samp>Javadocs</samp>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	public FinancialTransaction reverseApproval(
	    PluginContext        pluginContext,
	    FinancialTransaction financialTransaction,
	    boolean              retry)
		throws PluginException, RemoteException{
		
		return financialTransaction;
	}

	/**
	 * <p>
	 * This method cancels the specified credit.
	 * </p>
	 *
	 * <p>
	 * The amount can be partial if the plug-in supports partial amounts.  If the plug-in does not 
     * support partial amounts, then the {@link InvalidDataException} should be thrown.
	 * </p>
	 *
     * <p>
     * A <code>reverse credit</code> transaction is only requested against a plug-in if a previous
     * <code>credit</code> transaction has been requested and is either in 
     * {@link FinancialTransaction#STATE_SUCCESS} or {@link FinancialTransaction#STATE_FAILED}.  
     * </p>
     * 
     * @param pluginContext The plug-in context to be used in this request.
     * @param financialTransaction The details of the transaction to be processed.
     * @param retry Indicates if the same transaction has been tried before by the 
     *        Payment Plug-in Controller.
     * @return The <code>FinancialTransaction</code> with the results of the transaction.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <samp>Javadocs</samp>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	public FinancialTransaction reverseCredit(
	    PluginContext       pluginContext,
	    FinancialTransaction financialTransaction,
	    boolean              retry)
		throws PluginException, RemoteException{
		
		return financialTransaction;
	}

	/**
	 * <p>
	 * This method cancels a previously deposited amount.
	 * </p>
	 *
	 * <p>
	 * The amount can be partial if the plug-in supports partial amounts.  If the plug-in does not 
     * support partial amounts, then the {@link InvalidDataException} should be thrown.
	 * </p>
	 *
     * <p>
     * A <code>reverse deposit</code> transaction is only requested against a plug-in if a previous
     * <code>deposit</code> transaction has been requested and is either in 
     * {@link FinancialTransaction#STATE_SUCCESS} or {@link FinancialTransaction#STATE_FAILED}.  
     * </p>
     * 
     * @param pluginContext The plug-in context to be used in this request.
     * @param financialTransaction The details of the transaction to be processed.
     * @param retry Indicates if the same transaction has been tried before by the 
     *        Payment Plug-in Controller.
     * @return The <code>FinancialTransaction</code> with the results of the transaction.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <samp>Javadocs</samp>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	public FinancialTransaction reverseDeposit(
	    PluginContext       pluginContext,
	    FinancialTransaction financialTransaction,
	    boolean              retry)
		throws PluginException, RemoteException{
		
		return financialTransaction;
	}

	/**
	 * <p>
	 * This method validates the correctness and existence of any account associated with a
     * <code>PaymentInstruction</code>.
     * </p>
     * 
     * <p> This method may result in transactions to the back-end system. 
     * No funds should be transfered. This method goes beyond 
     * {@link #checkPaymentInstruction(PluginContext, PaymentInstruction)}.
	 * The intention of this method is to provide a thorough validation of 
     * the <code>PaymentInstruction</code> at an account level.
	 * </p>
	 *
	 * <p>
	 * Example: Verify if a credit line account exists in an accounting system.
	 * </p>
	 *
	 * <p>
	 * Some plug-ins might not be able to actually
     * validate the existence and correctness of an account without actually transfering funds.
     * Plug-ins might also implement this method using the same validation performed in
     * {@link #checkPaymentInstruction(PluginContext, PaymentInstruction)}. If this method is
     * not implemented, the Payment Plug-in Controller will invoke 
     * {@link #checkPaymentInstruction(PluginContext, PaymentInstruction)} to validate
     * a <code>PaymentInstruction</code>. If this method is also not implemented, then 
     * Payment Plug-in Controller will
     * assume that the <code>PaymentInstruction</code> is valid.
     * 
	 * </p>
     * @param pluginContext The plug-in context to be used in this request.
     * @param paymentInstruction Contains the payment method details required for the validation.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <samp>Javadocs</samp>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
     * 
     * @see #checkPaymentInstruction(PluginContext, PaymentInstruction).
	 */
	public void validatePaymentInstruction(
	    PluginContext     pluginContext,
	    PaymentInstruction paymentInstruction)
		throws PluginException, RemoteException{
		

	}
	
	/**
	 * <p>
	 * This method checks whether XML configuration for the Plugin is available. For the SimpleOffline, if the
	 * XML file is missing, or renamed, or its content is invalid, then false is returned. Otherwise
	 * true is returned. And for other Plugin, always true is returned since no corresponding XML
	 * configuration file exists.
	 * </p>
	 * <p>
	 * This API is used by PluginFactory in accelerator. When false is returned, PluginFactory will
	 * not put this plugin into the available collection. And further it will not be listed in the available
	 * payment methods in accelerator.
	 * </P>
	 * @return If the XML configuration for the Plugin is available.
	 * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	public boolean checkHealth() throws RemoteException{
		
		return true;
	}
	
	/**
	 * This method is called when the plugin is loaded for the first time.
	 * <p>
	 * This method starts a thread to read the response packet from the socket
	 * connecting to <samp>psp</samp> Host, retransmit the request that has been sent
	 * previously as well.
	 * @return if the online plugin works well
	 */
	public boolean initPlugin(PluginConfiguration pcl) throws RemoteException{
		return true;
	}
	
	/**
     * This method send the batch to the back ends.
     * @param pluginContext plugin context that contains the informations such as 
     *        the plugin configurations and the merchant's information.
     * @param batch the batch to be processed.
     * @param forceClose A boolean that indicates if the batch needs to be closed.
     * @return The updated batch value object.
     * @exception PluginException This exception is thrown if any error occurs.
     */
	public Batch settleBatch(PluginContext pluginContext, Batch batch, boolean forceClose) throws PluginException,RemoteException{
		return batch;
	}
	
	/**
	 * This method send <samp>RFR</samp> request to the back ends and update the transactions included in the batch
	 * and the batch state according to the response.
	 * @param pluginContext plugin context that contains the informations such as 
     *        the plugin configurations and the merchant's information.
	 * @param batch the batch needs to be gotten from the back ends.
	 * @return The updated batch value object.
	 * @exception PluginException This exception is thrown if any error occurs.
	 */
	public Batch getBatch(PluginContext pluginContext, Batch batch) throws PluginException,RemoteException{
		return batch;
	}
	
	/**
	 * <p>
	 * This method gets the available balance for the specified <code>PaymentInstruction</code> account.
     * </p>
     * 
     * <p>This methods is typically used for stored-value cards, gift cards, gift certificates
     * or credit line (or line of credit) implementations.
	 * </p>
	 *
	 * @param pluginContext The plug-in context to be used in this request.
	 * @param paymentInstruction The <code>PaymentInstruction</code> container for which the 
     *        balance needs to be obtained.
	 * @return The available amount that can be consumed in the payment instruction;
     *         <code>null</code> if the value cannot be retrieved.
	 * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <code>Javadocs</code>.
	 * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	public BigDecimal getAvailableBalance(
	    PluginContext      pluginContext,
		PaymentInstruction paymentInstruction)
		throws PluginException, RemoteException{
		
		return BigDecimal.ZERO;
	}

	/**
	 * <p>
     * This method refreshes the current attributes of the <code>Credit</code> container.  
     * </p>
     * 
     * <p> The Payment Plug-in controller provides a fully pre-populated <code>Credit</code>
     * container as input.  The plug-in determines which attributes have changed and
	 * updates those attributes on the returned object.
     * </p>
     * 
     * <p>
     * Note that the <code>Credit</code> parameter can be used as the object to be returned.
     * However, only the changes made on the actual returned <code>Credit</code> container 
     * object will be processed. Changes in the container parameter passed in will be ignored.
     * </p>
     * 
     * 
	 * @param pluginContext The plug-in context to be used in this request.
	 * @param credit The pre-populated <code>Credit</code> container.
	 * @return The updated <code>Credit</code> container object.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <code>Javadocs</code>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	public Credit getCredit(
		PluginContext pluginContext,
	    Credit        credit)
		throws PluginException, RemoteException{
		
		return credit;
	}

	/**
     * <p>
     * This method refreshes the current attributes of the <code>Payment</code> container.  
     * </p>
     * 
     * <p> The Payment Plug-in Controller provides a fully pre-populated <code>Payment</code>
     * container as input.  The plug-in determines which attributes have changed and
     * updates those attributes on the returned object.
     * </p>
     * 
     * <p>
     * Note that the <code>Payment</code> parameter can be used as the object to be returned.
     * However, only the changes made on the actual returned <code>Payment</code> container 
     * object will be processed. Changes in the container parameter passed in will be ignored.
     * </p>
     * 
     * 
     * @param pluginContext The plug-in context to be used in this request.
     * @param payment The pre-populated <code>Payment</code> container.
     * @return The updated <code>Payment</code> container object.
     * @exception PluginException Thrown when any exception conditions specified by the 
     *         <code>PluginException</code> hierarchy are met; for more details, read the
     *         plug-in exceptions hierarchy <code>Javadocs</code>.
     * @exception RemoteException Thrown by the EJB container as part of the EJB specification.
	 */
	public Payment getPayment(
	    PluginContext pluginContext,
		Payment       payment)
		throws PluginException, RemoteException{	
		
		return payment;
	}
}
